;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'mule '("edict"))
(custom-add-loads 'edict-update '("edict-update"))
(custom-add-loads 'dl '("dui"))
(custom-add-loads 'edict '("dui" "edict-edit" "edict-update" "edict"))

;;; custom-load.el ends here
